import os
import shutil

import numpy as np
import pandas as pd

d2m = 111194.92664455874
d2km = 111.19492664455874


def rotate_2d_points(points: np.ndarray, degree: float) -> np.ndarray:
    """
    Rotate a 2D point counterclockwise by a given number of degrees.

    :param points: np.array([[x1, y1],[x2,y2],...])
    :param degree: The angle in degrees by which the point is to be rotated (anti-clockwise).
    :return: rotated_points.
    """
    # Convert degrees to radians
    radians = np.radians(degree)

    # Rotation matrix for counterclockwise rotation
    rotation_matrix = np.array(
        [[np.cos(radians), -np.sin(radians)], [np.sin(radians), np.cos(radians)]]
    ).T

    # Rotated point
    rotated_points = np.dot(points, rotation_matrix)
    # print("rotated_points", rotated_points.shape)

    return rotated_points


def rotate_rtz_to_enz(az_in_deg, r, t, z):
    az = np.deg2rad(az_in_deg)
    e = r * np.sin(az) - t * np.cos(az)
    n = r * np.cos(az) + t * np.sin(az)
    return np.array([e, n, z])


def rotate_vector_x(v, gamma):
    """

    :param v:
    :param gamma: deg
    :return:
    """
    gamma = np.deg2rad(gamma)
    R = np.array(
        [
            [1, 0, 0],
            [0, np.cos(gamma), -np.sin(gamma)],
            [0, np.sin(gamma), np.cos(gamma)],
        ]
    )
    rotated_vec = np.dot(R, v)

    return rotated_vec


def rotate_vector_y(v, gamma):
    """

    :param v:
    :param gamma: deg
    :return:
    """
    gamma = np.deg2rad(gamma)
    R = np.array(
        [
            [np.cos(gamma), 0, np.sin(gamma)],
            [0, 1, 0],
            [-np.sin(gamma), 0, np.cos(gamma)],
        ]
    )

    rotated_vec = np.dot(R, v)

    return rotated_vec


def rotate_vector_z(v, gamma):
    """

    :param v:
    :param gamma: deg
    :return:
    """
    gamma = np.deg2rad(gamma)
    R = np.array(
        [
            [np.cos(gamma), -np.sin(gamma), 0],
            [np.sin(gamma), np.cos(gamma), 0],
            [0, 0, 1],
        ]
    )

    rotated_vec = np.dot(R, v)

    return rotated_vec


def spherical_2_cartesian(r, phi, theta):
    """

    :param r:
    :param phi: deg
    :param theta: deg
    :return:
    """
    phi, theta = np.deg2rad(phi), np.deg2rad(theta)
    x = r * np.sin(theta) * np.cos(phi)
    y = r * np.sin(theta) * np.sin(phi)
    z = r * np.cos(theta)
    return np.array([x, y, z])


def cartesian_2_spherical(x, y, z):
    """

    :param x:
    :param y:
    :param z:
    :return: r, phi, theta ( , rad, rad)
    """
    r = np.sqrt(x**2 + y**2 + z**2)
    theta = np.arccos(z / r)
    if x != 0:
        phi_x = np.arctan(y / x)
        if (x > 0) and (y >= 0):
            phi = phi_x
        elif (x < 0) and (y >= 0):
            phi = phi_x + np.pi
        elif (x < 0) and (y < 0):
            phi = phi_x + np.pi
        elif (x > 0) and (y < 0):
            phi = 2 * np.pi + phi_x
        else:
            raise ValueError
    else:
        if y > 0:
            phi = np.pi / 2
        elif y < 0:
            phi = 3 / 2 * np.pi
        else:
            phi = 0
    return r, phi, theta


def geo_2_r_earth(lat, lon, dep, r0=6371000):
    """

    :param lat: deg
    :param lon: deg
    :param dep: m
    :param r0: m
    :return: r_earth (unit m)
    """
    lat, lon = np.deg2rad(lat), np.deg2rad(lon)
    r_earth = np.array(
        [
            [
                (r0 - dep) * np.cos(lat) * np.cos(lon),
                (r0 - dep) * np.cos(lat) * np.sin(lon),
                (r0 - dep) * np.sin(lat),
            ]
        ]
    ).flatten()
    return r_earth


def r_earth_2_geo(r_earth, r0=6371000):
    """

    :param r_earth: m
    :param r0: m
    :return: lat, lon, depth (deg, deg, m)
    """
    r, lon, co_lat = cartesian_2_spherical(r_earth[0], r_earth[1], r_earth[2])
    depth = r0 - r
    lon = np.rad2deg(lon)
    if lon > 180:
        lon = lon - 360
    lat = np.rad2deg(np.pi / 2 - co_lat)
    return np.array([lat, lon, depth])


def convert_axis_delta_geo2ned(lat0, lon0, dep0, lat1, lon1, dep1):
    """

    :param lat0:
    :param lon0:
    :param dep0:
    :param lat1:
    :param lon1:
    :param dep1:
    :return: r (in ned axis), unit m
    """
    r_earth0 = geo_2_r_earth(lat0, lon0, dep0)
    r_earth1 = geo_2_r_earth(lat1, lon1, dep1)
    delta_r_earth = r_earth1 - r_earth0
    lat0 = np.deg2rad(lat0)
    lon0 = np.deg2rad(lon0)
    A = np.array(
        [
            [-np.sin(lat0) * np.cos(lon0), -np.sin(lat0) * np.sin(lon0), np.cos(lat0)],
            [-np.sin(lon0), np.cos(lon0), 0],
            [-np.cos(lat0) * np.cos(lon0), -np.cos(lat0) * np.sin(lon0), -np.sin(lat0)],
        ]
    )
    delta_r_ned = np.dot(A, delta_r_earth).flatten()
    return delta_r_ned


def convert_axis_delta_ned2geo(lat0, lon0, dep0, r_ned):
    """

    :param lat0: deg
    :param lon0: deg
    :param dep0: m
    :param r_ned: np.ndarray, m
    :return: lat, lon, depth (deg, deg, m)
    """
    r_ned = np.array(r_ned).flatten()
    r_earth0 = geo_2_r_earth(lat0, lon0, dep0)
    lat0, lon0 = np.deg2rad(lat0), np.deg2rad(lon0)
    A = np.array(
        [
            [-np.sin(lat0) * np.cos(lon0), -np.sin(lat0) * np.sin(lon0), np.cos(lat0)],
            [-np.sin(lon0), np.cos(lon0), 0],
            [-np.cos(lat0) * np.cos(lon0), -np.cos(lat0) * np.sin(lon0), -np.sin(lat0)],
        ]
    )
    delta_r_earth = np.dot(A.T, r_ned).flatten()
    r_earth1 = r_earth0 + delta_r_earth
    lat, lon, depth = r_earth_2_geo(r_earth1)
    return np.array([lat, lon, depth])


def convert_sub_faults_geo2ned(sub_faults, source_point, approximate=True):
    """
    :param sub_faults: lat(deg), lon(deg), dep(km)
    :param source_point: lat(deg), lon(deg), dep(km)
    :param approximate:
    :return: sub_faults_ned (unit m)
    """
    sub_faults_c = sub_faults.copy()
    source_point_c = source_point.copy()
    sub_faults_c[:, 2] = sub_faults_c[:, 2] * 1e3
    source_point_c[2] = source_point_c[2] * 1e3
    sub_faults_ned = np.zeros_like(sub_faults_c)
    if not approximate:
        for n in range(sub_faults_c.shape[0]):
            sub_faults_ned[n, :] = convert_axis_delta_geo2ned(
                *source_point_c.copy(), *sub_faults_c[n, :].copy()
            ).flatten()
    else:
        for n in range(sub_faults_c.shape[0]):
            x = (sub_faults_c[n, 0] - source_point_c[0]) * d2m
            y = (sub_faults_c[n, 1] - source_point_c[1]) * d2m
            z = sub_faults_c[n, 2] - source_point_c[2]
            sub_faults_ned[n, :] = np.array([x, y, z])
    return sub_faults_ned


def select_df_geo(df: pd.DataFrame, lat_range, lon_range, time_range) -> pd.DataFrame:
    """
    :param df: must have columns "lat","lon","time"
    :param lat_range: [lat_min, lat_max]
    :param lon_range: [lon_min, lon_max]
    :param time_range: [time_min, time_max] in format "2000-01-01T00:00:00Z"
    return df_filtered
    """
    time_range[0] = pd.to_datetime(time_range[0])
    time_range[1] = pd.to_datetime(time_range[1])
    df["time"] = pd.to_datetime(df["time"])
    df_filtered = df[
        (df["lon"] >= lon_range[0])
        & (df["lon"] <= 180)
        & (df["lon"] >= -180)
        & (df["lon"] <= lon_range[1])
        & (df["lat"] >= lat_range[0])
        & (df["lat"] <= lat_range[1])
        & (df["time"] >= time_range[0])
        & (df["time"] <= time_range[1])
    ]
    return df_filtered


def geographic_centroid(points):
    """
    Calculate the approximate geographic center.

    :param points: NumPy array of shape (N, 2), where each row is [latitude_deg, longitude_deg]
    :return: (centroid_lat_deg, centroid_lon_deg)
    """
    lat_rad = np.deg2rad(points[:, 0])
    lon_rad = np.deg2rad(points[:, 1])

    x = np.cos(lat_rad) * np.cos(lon_rad)
    y = np.cos(lat_rad) * np.sin(lon_rad)
    z = np.sin(lat_rad)

    x_mean = x.mean()
    y_mean = y.mean()
    z_mean = z.mean()

    centroid_lat_rad = np.arctan2(z_mean, np.sqrt(x_mean**2 + y_mean**2))
    centroid_lon_rad = np.arctan2(y_mean, x_mean)

    centroid_lat_deg = np.rad2deg(centroid_lat_rad)
    centroid_lon_deg = np.rad2deg(centroid_lon_rad)

    return centroid_lat_deg, centroid_lon_deg


def cal_max_dist_from_2d_points(A: np.ndarray, B: np.ndarray):
    """

    :param A: (m,2)
    :param B: (n,2)
    :return: max_distance
    """
    # Calculate the differences in each dimension (broadcasting)
    differences = A[:, np.newaxis, :] - B[np.newaxis, :, :]

    # Square the differences and sum across columns (to get squared distances)
    squared_distances = np.sum(differences**2, axis=2)

    # Take the square root to get Euclidean distances
    distances = np.sqrt(squared_distances)

    # Find the maximum distance
    max_distance = np.max(distances)
    return max_distance


def create_rotate_z_mat(gamma):
    """
    Generates a rotation matrix about the Z-axis.
    From y to x.
    Parameters:
        gamma : float
            Rotation angle in radians.

    Returns:
        R : numpy.ndarray
            A 3x3 rotation matrix.
    """
    R = np.array(
        [
            [np.cos(gamma), -np.sin(gamma), 0],
            [np.sin(gamma), np.cos(gamma), 0],
            [0, 0, 1],
        ]
    )
    return R


def rotate_symmetric_tensor_series(tensor, gamma):
    """
    Rotates a series of symmetric tensors without using an explicit loop.

    Parameters:
        tensor: numpy array of shape (n, 6)
            Each row is [xx, xy, xz, yy, yz, zz] representing a symmetric tensor.
        gamma: float
            Rotation angle around the z-axis (in radians) used to create the rotation matrix.

    Returns:
        rotated_tensor: numpy array of shape (n, 6)
            Rotated tensor components in the same order as the input.
    """
    # Create the 3x3 rotation matrix (assumed to be defined elsewhere).
    R = create_rotate_z_mat(gamma)
    n = tensor.shape[0]

    # Construct full symmetric matrices from the condensed tensor representation.
    A = np.empty((n, 3, 3), dtype=tensor.dtype)
    A[:, 0, 0] = tensor[:, 0]
    A[:, 0, 1] = tensor[:, 1]
    A[:, 0, 2] = tensor[:, 2]
    A[:, 1, 0] = tensor[:, 1]
    A[:, 1, 1] = tensor[:, 3]
    A[:, 1, 2] = tensor[:, 4]
    A[:, 2, 0] = tensor[:, 2]
    A[:, 2, 1] = tensor[:, 4]
    A[:, 2, 2] = tensor[:, 5]

    # Rotate each tensor using batch matrix multiplication:
    # Compute rotated_A = R.T @ A @ R for each tensor.
    rotated_A = np.einsum("ij,njk,kl->nil", R.T, A, R)

    # Extract the independent components from the rotated tensors.
    rotated_tensor = np.empty((n, 6), dtype=tensor.dtype)
    rotated_tensor[:, 0] = rotated_A[:, 0, 0]
    rotated_tensor[:, 1] = rotated_A[:, 0, 1]
    rotated_tensor[:, 2] = rotated_A[:, 0, 2]
    rotated_tensor[:, 3] = rotated_A[:, 1, 1]
    rotated_tensor[:, 4] = rotated_A[:, 1, 2]
    rotated_tensor[:, 5] = rotated_A[:, 2, 2]

    return rotated_tensor



import os
import math
import warnings

import numpy as np
import pandas as pd

from .utils import convert_earth_model_nd2inp, call_exe
from .qseis2025inp import s as str_inp


def create_dir_qseis2025(
        path_green,
        event_depth,
        receiver_depth,
        dist_range,
        delta_dist,
        N_each_group=500,
):
    sub_dir = str(
        os.path.join(path_green, "%.2f" % event_depth, "%.2f" % receiver_depth)
    )
    N_dist = math.ceil((dist_range[1] - dist_range[0]) / delta_dist) + 1
    N_dist_group = math.ceil(N_dist / N_each_group)
    for n in range(N_dist_group):
        os.makedirs(os.path.join(sub_dir, "%d_0" % n), exist_ok=True)
    return N_dist, N_dist_group


def create_inp_qseis2025(
        path_green,
        event_depth,
        receiver_depth,
        dist_range,
        delta_dist,
        N_dist,
        N_dist_group,
        N_each_group,
        time_window,
        sampling_interval,
        output_observables,
        slowness_int_algorithm=0,
        eps_estimate_wavenumber=1e-6,
        source_radius_ratio=0.05,
        slowness_window=None,
        time_reduction_velo=0,
        wavenumber_sampling_rate=12,
        anti_alias=0.01,
        free_surface=True,
        wavelet_duration=4,
        wavelet_type=1,
        flat_earth_transform=True,
        path_nd=None,
        earth_model_layer_num=None,
):
    dist_range = dist_range.copy()
    output_observables = output_observables.copy()
    path_sub_dir = str(
        os.path.join(path_green, "%.2f" % event_depth, "%.2f" % receiver_depth)
    )
    # when receiver depth is not 0, change dists in inp file
    if flat_earth_transform:
        r_ratio = (6371 - receiver_depth) / 6371
    else:
        r_ratio = 1
    lines = str_inp.split("\n")
    lines = [line + "\n" for line in lines]

    lines_earth = lines[209:-22]
    lines_end = lines[-22:]

    # SOURCE PARAMETERS
    lines[26] = "%.2f\n" % event_depth

    # RECEIVER PARAMETERS
    lines[42] = "%.2f\n" % receiver_depth
    lines[43] = "1 1\n"
    lines[46] = "%f %f %d\n" % (
        0.0,
        time_window,
        round(time_window / sampling_interval + 1),
    )
    lines[47] = "%d %f\n" % (1, time_reduction_velo)

    # WAVENUMBER INTEGRATION PARAMETERS
    lines[73] = "%d\n" % slowness_int_algorithm
    lines[74] = "%f %f\n" % (eps_estimate_wavenumber, source_radius_ratio)
    if slowness_window is not None:
        lines[75] = "%f %f %f %f\n" % (
            slowness_window[0],
            slowness_window[1],
            slowness_window[2],
            slowness_window[3],
        )
    else:
        lines[75] = "0.0 0.0 0.0 0.0\n"
    lines[76] = "%f\n" % wavenumber_sampling_rate
    lines[77] = "%f\n" % anti_alias

    # OPTIONS FOR PARTIAL SOLUTIONS
    if free_surface:
        lines[111] = "0\n"
    else:
        lines[111] = "1\n"

    # SOURCE TIME FUNCTION (WAVELET) PARAMETERS (Note 3)
    lines[132] = "%d %d\n" % (wavelet_duration, wavelet_type)

    # OUTPUT FILES FOR GREEN'S FUNCTIONS (Note 4)
    lines[181] = " ".join(["%d" % output_observables[_] for _ in range(5)]) + "\n"

    # GLOBAL MODEL PARAMETERS (Note 5)
    if flat_earth_transform:
        lines[217] = "1\n"
    else:
        lines[217] = "0\n"

    if path_nd is not None:
        lines_earth = convert_earth_model_nd2inp(
            path_nd=path_nd, path_output="earth_model.dat"
        )
    if earth_model_layer_num is None:
        earth_model_layer_num = len(lines_earth)
    else:
        lines_earth = lines_earth[:earth_model_layer_num]
    lines[226] = "%d\n" % earth_model_layer_num
    lines = lines[:227] + lines_earth + lines_end

    for n in range(N_dist_group - 1):
        lines[44] = "%d\n" % N_each_group
        lines[45] = "%f %f\n" % (
            (dist_range[0] + n * N_each_group * delta_dist) * r_ratio,
            (dist_range[0] + ((n + 1) * N_each_group - 1) * delta_dist) * r_ratio,
        )
        path_inp = os.path.join(path_sub_dir, "%d_0" % n, "grn.inp")
        with open(path_inp, "w") as fw:
            fw.writelines(lines)
    else:
        res = N_dist - (N_dist_group - 1) * N_each_group
        lines[44] = "%d\n" % res
        lines[45] = "%f %f\n" % (
            (dist_range[0] + (N_dist_group - 1) * N_each_group * delta_dist) * r_ratio,
            dist_range[1] * r_ratio,
        )
        path_inp = os.path.join(path_sub_dir, "%d_0" % (N_dist_group - 1), "grn.inp")
        with open(path_inp, "w") as fw:
            fw.writelines(lines)
    return path_inp


def call_qseis2025(
        event_depth, receiver_depth, n_group, path_green, check_finished=False
):
    sub_sub_dir = str(
        os.path.join(
            path_green,
            "%.2f" % event_depth,
            "%.2f" % receiver_depth,
            "%d_0" % n_group,
        )
    )
    os.chdir(sub_sub_dir)
    path_inp = os.path.join(sub_sub_dir, "grn.inp")
    path_finished = os.path.join(sub_sub_dir, ".finished")

    if (
            check_finished
            and os.path.exists(path_finished)
            and len(os.listdir(sub_sub_dir)) > 2
    ):
        with open(path_finished, "r") as fr:
            output = fr.readlines()
        return output

    output = call_exe(
        path_green=path_green,
        path_inp=path_inp,
        path_finished=path_finished,
        name="qseis2025",
    )
    return output


def convert_pd2bin_qseis2025(path_greenfunc, remove=False):
    for com in [
        "tr",
        "tz",
        "tv",
        "ezz",
        "ezr",
        "err",
        "ett",
        "szz",
        "szr",
        "srr",
        "stt",
        "ot",
    ]:
        time_series_com = []
        for stype in ["ex", "ss", "ds", "cl"]:
            path_ascii = os.path.join(path_greenfunc, "%s.%s" % (stype, com))
            if not os.path.exists(path_ascii):
                # warnings.warn("ascii file %s do not exist, skip" % path_ascii)
                continue
            stype_com = pd.read_csv(path_ascii, sep="\\s+").to_numpy()
            time_series_com.append(stype_com[:, 1:])
            if remove:
                os.remove(path_ascii)
        if len(time_series_com) == 4:
            output_data = np.concatenate(
                [time_series_com[_] for _ in range(4)], dtype=np.float32
            )
            np.save(str(os.path.join(path_greenfunc, "grn_%s.npy" % com)), output_data)

    for com in ["tt", "ezt", "ert", "szt", "srt", "oz", "or"]:
        time_series_com = []
        for stype in ["ss", "ds"]:
            path_ascii = os.path.join(path_greenfunc, "%s.%s" % (stype, com))
            if not os.path.exists(path_ascii):
                # warnings.warn("ascii file %s do not exist, skip" % path_ascii)
                continue
            stype_com = pd.read_csv(path_ascii, sep="\\s+").to_numpy()
            time_series_com.append(stype_com[:, 1:])
            if remove:
                os.remove(path_ascii)
        if len(time_series_com) == 2:
            output_data = np.concatenate(
                [time_series_com[_] for _ in range(2)], dtype=np.float32
            )
            np.save(os.path.join(path_greenfunc, "grn_%s.npy" % com), output_data)


if __name__ == "__main__":
    pass



s = """#=============================================================================
# This is the input file of FORTRAN77 program "edgrn" for calculating
# the Green's functions of a layered elastic half-space earth model. All
# results will be stored in the given directory and provide the necessary
# data base for the program "edcmp" for computing elastic deformations
# (3 displacement components, 6 strain/stress components and 2 vertical tilt
# components) induced by a general dislocation source.
#
# For questions please contact with e-mail to "wang@gfz-potsdam.de".
#
# First implemented in May, 1997
# Last modified: Nov, 2001
#
# by Rongjiang Wang
# GeoForschungsZetrum Potsdam, Telegrafenberg, 14473 Potsdam, Germany
#
# For questions and suggestions please send e-mails to wang@gfz-potsdam.de
#
#------------------------------------------------------------------------------
#
#	PARAMETERS FOR THE OBSERVATION PROFILE
#	======================================
# 1. the uniform depth of the observation points [m]
# 2. number of the equidistant radial distances (max. = nrmax in edgglobal.h),
#    the start and end of the distances [m]
# 3. number of the equidistant source depths (max. = nzsmax in edgglobal.h),
#    the start and end of the source depths [m]
#
#    If possible, please choose the observation depth either significantly
#    different from the source depths or identical with one of them.
#
#    The 2D distance and depth grids defined here should be necessarily large
#    and dense enough for the discretisation of the real source-observation
#    configuration to be considered later.
#
#    r1,r2 = minimum and maximum horizontal source-observation distances
#    z1,z2 = minimum and maximum source depths
#
#------------------------------------------------------------------------------
0.00d+00                           |dble: obs_depth;
201  0.00d+00   100.00d+03         |int: nr; dble: r1, r2;
40  0.25d+03    19.75d+03          |int: nzs; dble: zs1, zs2;
#------------------------------------------------------------------------------
#
#	WAVENUMBER INTEGRATION PARAMETERS
#	=================================
# 1. sampling rate for wavenumber integration (the ratio between the Nyquist
#    wavenumber and the really used wavenumber sample; the suggested value is
#    10-128: the larger this value is chosen, the more accurate are the results
#    but also the more computation time will be required)
#------------------------------------------------------------------------------
12.0                            |dble: srate;
#------------------------------------------------------------------------------
#
#	OUTPUT FILES
#	============
#
# 1. output directory, the three file names for fundamental Green's functions
#    Note that all file or directory names should not be longer than 80
#    characters. Directories must be ended by (unix) or (dos)!
#------------------------------------------------------------------------------
''  'edgrn.ss'  'edgrn.ds'  'edgrn.cl'  |char: outputs,grnfile(3);
#------------------------------------------------------------------------------
#
#	MULTILAYERED MODEL PARAMETERS
#	=============================
# The interfaces at which the elastic parameters are continuous, the surface
# and the upper boundary of the half-space are all defined by a single data
# line; The interfaces, at which the elastic parameters are discontinuous,
# are all defined by two data lines. This input format would also be needed for
# a graphic plot of the layered model.
#
# Layers which have different upper and lower parameter values, will be treated
# as layers with a constant gradient, and will be discretised by a number of
# homogeneous sublayers. Errors due to the discretisation are limited within
# about 5%.
#
# 1. total number of the data lines (max. = lmax in edgglobal.h)
# 2. table for the layered model
#------------------------------------------------------------------------------
6                               |int: no_model_lines;
#------------------------------------------------------------------------------
#    no  depth[m]       vp[m/s]         vs[m/s]        ro[kg/m^3]
#------------------------------------------------------------------------------
1  0.00000            6000.0             3520.0             2720.0
2  19060.0            6000.0             3520.0             2720.0
3  19060.0            6300.0             3680.0             2790.0
4  28200.0            6300.0             3680.0             2790.0
5  28200.0            6600.0             3820.0             2850.0
6  38110.0            6600.0             3820.0             2850.0
#=======================end of input=========================================="""



from pygrnwang.create_spgrn_bulk import *

if __name__ == '__main__':
    sampling_interval = 1/8
    time_window = 256-sampling_interval
    path_green = "/e/grns_test/test_spgrn_%f" % sampling_interval
    os.makedirs(path_green, exist_ok=True)
    pre_process_spgrn2020(
        processes_num=1,
        path_green=path_green,
        path_bin="/home/zjc/python_works/pygrnwang/pygrnwang/exec/spgrn2020.bin",
        event_depth_list=[10],
        receiver_depth_list=[1],
        spec_time_window=time_window,
        sampling_interval=sampling_interval,
        max_frequency=0.2,
        max_slowness=0.4,
        anti_alias=0.01,
        gravity_fc=0,
        gravity_harmonic=0,
        cal_sph=1,
        cal_tor=1,
        source_radius=0,
        cal_gf=1,
        time_window=time_window,
        green_before_p=20,
        source_duration=0,
        dist_range=[0, 300],
        delta_dist_range=[100, 100],
        path_nd='/home/zjc/python_works/pygrnwang/test/ak135fc.nd',
        earth_model_layer_num=12,
        physical_dispersion=0,
    )
    create_grnlib_spgrn2020_parallel(
        path_green=path_green, check_finished=False
    )
